@@ -9,9 +9,12 @@ import javassist.bytecode.AttributeInfo;
 import javassist.bytecode.ClassFile;
 import javassist.bytecode.ConstPool;
 import javassist.bytecode.InnerClassesAttribute;
+import javassist.bytecode.NestHostAttribute;
+import javassist.bytecode.NestMembersAttribute;
 import javassist.expr.ExprEditor;
 import javassist.expr.Handler;
 import javassist.expr.MethodCall;
+import javassist.expr.NewExpr;
 
 @SuppressWarnings({"rawtypes","unchecked","unused"})
 public class JvstTest5 extends JvstTestRoot {
@@ -221,7 +224,7 @@ public class JvstTest5 extends JvstTestRoot {
                 "}");
         System.out.println(src);
         badClass.addMethod(CtMethod.make(src, badClass));
-        Class clazzz = badClass.toClass();
+        Class clazzz = badClass.toClass(Class.forName("DefineClassCapability"));
         Object obj = clazzz.getConstructor().newInstance(); // <-- falls here
     }
 
@@ -416,4 +419,104 @@ public class JvstTest5 extends JvstTestRoot {
         Object obj = make(cc.getName());
         assertEquals(1, invoke(obj, "test"));
     }
+
+    public void testNestHostAttribute() throws Exception {
+        CtClass cc = sloader.get("test5.NestHost$Foo");
+        ClassFile cf = cc.getClassFile();
+        NestHostAttribute attr = (NestHostAttribute)cf.getAttribute(NestHostAttribute.tag);
+        assertEquals(test5.NestHost.class.getName(),
+                     cf.getConstPool().getClassInfo(attr.hostClassIndex()));
+    }
+
+    public void testNestMembersAttribute() throws Exception {
+        CtClass cc = sloader.get("test5.NestHost");
+        ClassFile cf = cc.getClassFile();
+        NestMembersAttribute attr = (NestMembersAttribute)cf.getAttribute(NestMembersAttribute.tag);
+        assertEquals(2, attr.numberOfClasses());
+        String[] names = new String[2];
+        for (int i = 0; i < 2; i++)
+            names[i] = cf.getConstPool().getClassInfo(attr.memberClass(i));
+        
+        assertFalse(names[0].equals(names[1]));
+        assertTrue(names[0].equals("test5.NestHost$Foo") || names[0].equals("test5.NestHost$Bar"));
+        assertTrue(names[1].equals("test5.NestHost$Foo") || names[1].equals("test5.NestHost$Bar"));
+    }
+
+    public void testNestMembersAttributeCopy() throws Exception {
+        CtClass cc = sloader.get("test5.NestHost2");
+        cc.getClassFile().compact();
+        cc.writeFile();
+        make(cc.getName());
+    }
+
+    public void testNestHostAttributeCopy() throws Exception {
+        CtClass cc = sloader.get("test5.NestHost2$Foo");
+        cc.getClassFile().compact();
+        cc.toClass(test5.DefineClassCapability.class);
+    }
+
+    public void testSwitchCaseWithStringConstant() throws Exception {
+        CtClass cc = sloader.get("test5.SwitchCase");
+        cc.addMethod(CtNewMethod.make(
+                "public int run() {" +
+                "    String s = \"foobar\";\n" +
+                "    switch (s) {\n" +
+                "    case STR1: return 1;\n" +
+                "    case \"foobar\": return 2;\n" +
+                "    default: return 3; }\n" +
+                "}\n", cc));
+        cc.writeFile();
+        Object obj = make(cc.getName());
+        assertEquals(2, invoke(obj, "run"));   
+    }
+
+    public void testNestPrivateConstructor() throws Exception {
+        CtClass cc = sloader.get("test5.NestHost3$Builder");
+        cc.instrument(new ExprEditor() {
+            public void edit(NewExpr e) throws CannotCompileException {
+                String code = "$_ = $proceed($$);";
+                e.replace(code);
+            }
+        });
+        cc.writeFile();
+        try {
+            Class<?> nestHost3Class = cloader.loadClass("test5.NestHost3");
+            Object builder = nestHost3Class.getDeclaredMethod("builder").invoke(nestHost3Class);
+            Class<?> nestHost3BuilderClass = cloader.loadClass("test5.NestHost3$Builder");
+            nestHost3BuilderClass.getDeclaredMethod("build").invoke(builder);
+        } catch (Exception ex) {
+            ex.printStackTrace();
+            fail("it should be able to access the private constructor of the nest host");
+        }
+    }
+
+    public void testSwitchCaseWithStringConstant2() throws Exception {
+        CtClass cc = sloader.makeClass("test5.SwitchCase2");
+        cc.addMethod(CtNewMethod.make(
+                "public int run() {" +
+                "    String s = \"foo\";\n" +
+                "    switch (s) {\n" +
+                "    case test5.SwitchCase.STR1: return 1;\n" +
+                "    case \"foobar\": return 2;\n" +
+                "    }\n" +
+                "    return 3;\n" +
+                "}\n", cc));
+        cc.writeFile();
+        Object obj = make(cc.getName());
+        assertEquals(1, invoke(obj, "run"));   
+    }
+
+    // Issue #241
+    public void testInsertBeforeAndDollarR() throws Exception {
+        CtClass cc = sloader.get(test5.InsertBeforeDollarR.class.getName());
+        CtMethod m = cc.getDeclaredMethod("foo");
+        m.insertBefore("{ if ($1 == 1) return ($r)$2; }");
+        try {
+            m.insertBefore("{ $_ = \"bar\"; }");
+            assertTrue(false);
+        } catch (CannotCompileException e) {}
+        cc.writeFile();
+        Object obj = make(cc.getName());
+        assertEquals(1, invoke(obj, "run"));
+    }
 }
